@@ -58,7 +58,7 @@ def update_glfw_canvasses():
     # only raise errors if the logging system fails.
     canvases = tuple(all_glfw_canvases)
     for canvas in canvases:
-        if canvas._need_draw:
+        if canvas._need_draw and not canvas._is_minimized:
             canvas._need_draw = False
             canvas._draw_frame_and_present()
     return len(canvases)
@@ -151,6 +151,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         self._need_draw = False
         self._request_draw_timer_running = False
         self._changing_pixel_ratio = False
+        self._is_minimized = False
 
         # Register ourselves
         all_glfw_canvases.add(self)
@@ -163,6 +164,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         glfw.set_window_focus_callback(self._window, self._on_window_dirty)
         set_window_content_scale_callback(self._window, self._on_pixelratio_change)
         set_window_maximize_callback(self._window, self._on_window_dirty)
+        glfw.set_window_iconify_callback(self._window, self._on_iconify)
 
         # User input
         self._key_modifiers = set()
@@ -199,12 +201,15 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
     def _on_close(self, *args):
         all_glfw_canvases.discard(self)
         glfw.hide_window(self._window)
-        self.handle_event({"event_type": "close"})
+        self._handle_event_and_flush({"event_type": "close"})
 
     def _on_window_dirty(self, *args):
         self._request_draw()
 
-    # Helpers
+    def _on_iconify(self, window, iconified):
+        self._is_minimized = bool(iconified)
+
+    # helpers
 
     def _mark_ready_for_draw(self):
         self._request_draw_timer_running = False
@@ -229,7 +234,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
             "height": self._logical_size[1],
             "pixel_ratio": self._pixel_ratio,
         }
-        self.handle_event(ev)
+        self._handle_event_and_flush(ev)
 
     def _set_logical_size(self, new_logical_size):
         # There is unclarity about the window size in "screen pixels".
@@ -353,7 +358,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
         }
 
         # Emit the current event
-        self.handle_event(ev)
+        self._handle_event_and_flush(ev)
 
         # Maybe emit a double-click event
         self._follow_double_click(action, button)
@@ -405,7 +410,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
                 "ntouches": 0,  # glfw dows not have touch support
                 "touches": {},
             }
-            self.handle_event(ev)
+            self._handle_event_and_flush(ev)
 
     def _on_cursor_pos(self, window, x, y):
         # Store pointer position in logical coordinates
@@ -478,7 +483,7 @@ class GlfwWgpuCanvas(WgpuAutoGui, WgpuCanvasBase):
             "key": keyname,
             "modifiers": list(self._key_modifiers),
         }
-        self.handle_event(ev)
+        self._handle_event_and_flush(ev)
 
 
 # Make available under a name that is the same for all gui backends
