@@ -171,8 +171,8 @@ def compute_with_buffers(input_arrays, output_arrays, shader, n=None):
     compute_pass = command_encoder.begin_compute_pass()
     compute_pass.set_pipeline(compute_pipeline)
     compute_pass.set_bind_group(0, bind_group, [], 0, 999999)  # last 2 args not used
-    compute_pass.dispatch(nx, ny, nz)
-    compute_pass.end_pass()
+    compute_pass.dispatch_workgroups(nx, ny, nz)
+    compute_pass.end()
     device.queue.submit([command_encoder.finish()])
 
     # Read the current data of the output buffers
